import 'codec.dart';
import 'utils.dart';

int genNowMsgID() {
  return DateTime.now().microsecondsSinceEpoch ~/ 10000;
}

class Packet {
  final List<int> to;
  late final int id;
  int nonce = 0;
  final List<int> from;
  final List<int> data;

  Packet(this.to, this.from, this.data, [this.nonce = 0, int id = -1]) {
    this.id = id < 0 ? genNowMsgID() : id;
  }

  static Packet decode(List<int> b) {
    var parts = Codec.decode(b);
    return Packet(parts[0], parts[2], parts[3],
        Utils.parseBinaryToInt(parts[4]), Utils.parseBinaryToInt(parts[1]));
  }

  @override
  String toString() {
    return {
      "to": to,
      "from": from,
      "id": id,
      "data": data,
      "nonde": nonce,
    }.toString();
  }

  List<int> encode() {
    return Codec.encode(
        [to, Utils.int64ToBinary(id), from, data, Utils.int64ToBinary(nonce)]);
  }
}
