import 'dart:io';
import 'dart:typed_data';

import 'package:http/http.dart' as http;
import 'net_addr.dart';

class Tail {
  final RawDatagramSocket _socket;
  late final Stream<Uint8List> stream;
  final NetAddr host;

  static Future<Tail> connect(NetAddr host) async {
    return Tail(await RawDatagramSocket.bind("0.0.0.0", 0), host);
  }

  Tail(this._socket, this.host) {
    stream = _socket
        .map((event) {
          return _socket.receive()?.data;
        })
        .where((event) => event != null)
        .map((event) => event!);
  }

  Future<void> check(List<int> last) async {
    try {
      _socket.send(last, (await host.lookup()).first, host.port);
    } catch (_) {}
  }

  void close() {
    return _socket.close();
  }

  static Future<void> push(NetAddr addr, List<int> data) async {
    var resp = await http.post(addr.toUri(scheme: "http"), body: data);
    if (resp.statusCode != 204) {
      throw Exception(resp.body.isEmpty ? resp.statusCode : resp.body);
    }
  }
}
